#include <cstdlib>
#include <new>

#define MEM_OFFSET 16

#if (__has_feature(cxx_noexcept))
#  define EXCEPT1
#  define EXCEPT2 noexcept
#else
#  define EXCEPT1 throw(std::bad_alloc)
#  define EXCEPT2 throw()
#endif

namespace
{
    void* allocMem(std::size_t size)
    {
        void* p = malloc(size+MEM_OFFSET);

        return (char*)p + MEM_OFFSET;
    }

    void deallocMem(void* p)
    {
        if(p)
        {
            free((char*)p - MEM_OFFSET);
        }
    }
}

void* operator new(std::size_t size) EXCEPT1
{
    return allocMem(size);
}

void* operator new(size_t size, const std::nothrow_t&) EXCEPT2
{
    return allocMem(size);
}

void  operator delete(void* ptr) EXCEPT2
{
    return deallocMem(ptr);
}

void  operator delete(void* ptr, size_t size) EXCEPT2
{
    return deallocMem(ptr);
}

void  operator delete(void* ptr, const std::nothrow_t&) EXCEPT2
{
    return deallocMem(ptr);
}

void* operator new[](size_t size) EXCEPT1
{
    return allocMem(size);
}

void* operator new[](size_t size, const std::nothrow_t&) EXCEPT2
{
    return allocMem(size);
}

void operator delete[](void* ptr) EXCEPT2
{
    return deallocMem(ptr);
}

void operator delete[](void* ptr, std::size_t size) EXCEPT2
{
    return deallocMem(ptr);
}